#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>
#include <set>
#include <queue>
#include <list>
#include <fstream>

typedef std::vector<int> VI;
typedef std::vector<VI> VVI;
typedef std::pair<int, int> II;
typedef std::vector<II> VII;
typedef std::vector<VII> VVII;

typedef std::list<int> LI;
typedef std::vector<LI> VLI;
typedef std::set<int> SI;

typedef std::map<II, int> MII_I;

int main(int argc, char *argv[]) {
  std::string program_out_path = argv[1];
  std::string judge_out_path = argv[2];
  std::string input_path = argv[3];

  std::fstream f(input_path.c_str());

  int N, M, P;
  f >> N >> M >> P;

  VI cash(N+1, 0);
  MII_I debts, init_debts;
  for(int i = 0; i < M; i ++) {
    int a, b, c;
    f >> a >> b >> c;
    debts[II(a,b)] = c;
  }

  MII_I loans;
  for(int i = 0; i < P; i ++) {
    int a, b, c;
    f >> a >> b >> c;
    loans[II(a,b)] = c;
  }

  std::ifstream fj(judge_out_path.c_str());
  std::ifstream fp(program_out_path.c_str());

  // same verdict?
  std::string j_v;
  fj >> j_v;
  std::string p_v;
  fp >> p_v;

  if(j_v != p_v) {
    std::cout << "possible vs. impossible" << std::endl;
    return 1;
  }

  // if impossible && agree, done
  if(j_v == "Impossible") return 0;

  // otherwise, tracing time
  int a, b, c;
  int count = 0;
  while(fp >> a >> b >> c) {
    count++;

    // sanity check
    if (!(1 <= a && a <= N && 1 <= b && b <= N && 1 <= c)) {
      std::cout << "sanity check" << std::endl;
      return 1;
    }
    // should be a debt before we pay, and we have to have cash
    if((debts.count(II(a,b)) + init_debts.count(II(a,b))) != 0 &&
       cash[a] >= c) {
	
      int debts_red = std::min(debts[II(a,b)], c);
      int idebts_red = std::min(init_debts[II(a,b)], c - debts_red);

      // pay debts from a->b
      debts[II(a,b)] -= debts_red;
      init_debts[II(a,b)] -= idebts_red;
	
      // can't go below zero
      if(debts_red + idebts_red != c) {
	std::cout << "paying too much on payment " << count << std::endl; 
	std::cout << "a, b, c = " << a << ' ' << b << ' ' << c << std::endl;
	std::cout << "debt a->b was " << debts[II(a,b)]  << std::endl;
	std::cout << "initial debt a->b was " << init_debts[II(a,b)]  << std::endl;
	return 1;
      }
      cash[a] -= c;
      cash[b] += debts_red;
    } else {
      // could be a lending, though
	
      // can we lend c units from a to b?
      if(loans[II(a,b)] >= c) {
	loans[II(a,b)] -= c;
	init_debts[II(b,a)] += c;
	cash[b] += c;   // injects cash into b
      }
      // o/w, nope
      else {
	std::cout << "cannot loan any more from " << a << " to " << b
		  << std::endl;
	return 1;
      }
    }
  }

  // are all debts zero?
  for(MII_I::iterator i = debts.begin(); i != debts.end(); ++i) {
    if(i->second != 0) {
      std::cout << "debt for " << i->first.first << " to " << i->first.second
		<< " not cleared: " << i->second << " left" << std::endl;
      return 1;
    }
  }

  for(MII_I::iterator i = init_debts.begin(); i != init_debts.end(); ++i) {
    if(i->second != 0) {
      std::cout << "initial debt for " << i->first.first << " to " << i->first.second
		<< " not cleared: " << i->second << " left" << std::endl;
      return 1;
    }
  }

  return 0;
}
